@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,7 @@
 
 /* guru interface: requires care in alignment, r - i, etcetera. */
 void X(execute_dft_r2c)(const X(plan) p, R *in, C *out)
-{
+WITH_ALIGNED_STACK({
      plan_rdft2 *pln = (plan_rdft2 *) p->pln;
-     problem_rdft2 *prb = (problem_rdft2 *) p->prb;
-     pln->apply((plan *) pln, in, in + (prb->r1 - prb->r0), out[0], out[0]+1);
-}
+     pln->apply((plan *) pln, in, out[0], out[0]+1);
+})
