@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2006 Matteo Frigo
- * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,11 +18,16 @@
  *
  */
 
-/* $Id: kdft-dit.c,v 1.14 2006-01-05 03:04:26 stevenj Exp $ */
+/* $Id: kdft-dit.c,v 1.9 2003/03/15 20:29:42 stevenj Exp $ */
 
-#include "ct.h"
+#include "dft.h"
 
-void X(kdft_dit_register)(planner *p, kdftw codelet, const ct_desc *desc)
+void (*X(kdft_dit_register_hook))(planner *, kdft_dit, const ct_desc *) = 0;
+
+void X(kdft_dit_register)(planner *p, kdft_dit codelet, const ct_desc *desc)
 {
-     X(regsolver_ct_directw)(p, codelet, desc, DECDIT);
+     REGISTER_SOLVER(p, X(mksolver_dft_ct_dit)(codelet, desc));
+     REGISTER_SOLVER(p, X(mksolver_dft_ct_ditbuf)(codelet, desc));
+     if (X(kdft_dit_register_hook))
+	  X(kdft_dit_register_hook)(p, codelet, desc);
 }
