@@ -1,7 +1,7 @@
 (*
  * Copyright (c) 1997-1999 Massachusetts Institute of Technology
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
  *
  *)
 
+(* $Id: algsimp.ml,v 1.7 2003/03/15 20:29:42 stevenj Exp $ *)
+let cvsid = "$Id: algsimp.ml,v 1.7 2003/03/15 20:29:42 stevenj Exp $"
 
 open Util
 open Expr
@@ -59,10 +61,6 @@ end = struct
     | (Times (a, a'), Times (b, b')) ->
  	((a == b) && (a' == b')) or
  	((a == b') && (a' == b))
-    | (CTimes (a, a'), CTimes (b, b')) ->
- 	((a == b) && (a' == b')) or
- 	((a == b') && (a' == b))
-    | (CTimesJ (a, a'), CTimesJ (b, b')) -> ((a == b) && (a' == b'))
     | (Plus a, Plus b) -> subset a b && subset b a
     | (Uminus a, Uminus b) -> (a == b)
     | _ -> false
@@ -115,10 +113,6 @@ end = struct
   and stimesM = function 
     | (Uminus a, b) -> stimesM (a, b) >>= suminusM
     | (a, Uminus b) -> stimesM (a, b) >>= suminusM
-    | (NaN I, CTimes (a, b)) -> stimesM (NaN I, b) >>= 
-	fun ib -> sctimesM (a, ib)
-    | (NaN I, CTimesJ (a, b)) -> stimesM (NaN I, b) >>= 
-	fun ib -> sctimesjM (a, ib)
     | (Num a, Num b) -> snumM (Number.mul a b)
     | (Num a, Times (Num b, c)) -> 
 	snumM (Number.mul a b) >>= fun x -> stimesM (x, c)
@@ -129,16 +123,6 @@ end = struct
 	stimesM (b, a)
     | (a, b) -> makeNode (Times (a, b))
 
-  and sctimesM = function 
-    | (Uminus a, b) -> sctimesM (a, b) >>= suminusM
-    | (a, Uminus b) -> sctimesM (a, b) >>= suminusM
-    | (a, b) -> makeNode (CTimes (a, b))
-
-  and sctimesjM = function 
-    | (Uminus a, b) -> sctimesjM (a, b) >>= suminusM
-    | (a, Uminus b) -> sctimesjM (a, b) >>= suminusM
-    | (a, b) -> makeNode (CTimesJ (a, b))
-
   and reduce_sumM x = match x with
     [] -> returnM []
   | [Num a] -> 
@@ -367,17 +351,9 @@ end = struct
  	| Plus a -> 
  	    mapM algsimpM a >>= splusM
  	| Times (a, b) -> 
- 	    (algsimpM a >>= fun a' ->
+ 	    algsimpM a >>= fun a' ->
  	      algsimpM b >>= fun b' ->
- 		stimesM (a', b'))
- 	| CTimes (a, b) -> 
- 	    (algsimpM a >>= fun a' ->
- 	      algsimpM b >>= fun b' ->
-		sctimesM (a', b'))
- 	| CTimesJ (a, b) -> 
- 	    (algsimpM a >>= fun a' ->
- 	      algsimpM b >>= fun b' ->
-		sctimesjM (a', b'))
+ 		stimesM (a', b')
  	| Uminus a -> 
  	    algsimpM a >>= suminusM 
  	| Store (v, a) ->
@@ -420,8 +396,6 @@ module Transpose = struct
 	    | Store (v, n) -> [n]
 	    | Plus l -> l
 	    | Times (a, b) -> [a; b]
-	    | CTimes (a, b) -> [a; b]
-	    | CTimesJ (a, b) -> [a; b]
 	    | Uminus x -> [x]
 	    | _ -> []
 	    in let rec loop t = function
@@ -447,12 +421,6 @@ module Transpose = struct
       | Times (a, b) when b == node -> 
 	  dualM candidate_parent >>= fun x' -> 
 	    returnM [makeTimes (a, x')]
-      | CTimes (a, b) when b == node -> 
-	  dualM candidate_parent >>= fun x' -> 
-	    returnM [CTimes (a, x')]
-      | CTimesJ (a, b) when b == node -> 
-	  dualM candidate_parent >>= fun x' -> 
-	    returnM [CTimesJ (a, x')]
       | Uminus n when n == node -> 
 	  dualM candidate_parent >>= fun x' -> 
 	    returnM [makeUminus x']
@@ -537,8 +505,6 @@ end = struct
 	    | Times _ -> (load, store, plus, times + 1, uminus, num)
 	    | Uminus _ -> (load, store, plus, times, uminus + 1, num)
 	    | Num _ -> (load, store, plus, times, uminus, num + 1)
-	    | CTimes _ -> (load, store, plus, times, uminus, num)
-	    | CTimesJ _ -> (load, store, plus, times, uminus, num)
 	    | NaN _ -> (load, store, plus, times, uminus, num))
 	    rest
     in let (l, s, p, t, u, n) = 
