@@ -1,7 +1,7 @@
 (*
  * Copyright (c) 1997-1999 Massachusetts Institute of Technology
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *)
+(* $Id: trig.ml,v 1.7 2003/03/25 16:51:49 stevenj Exp $ *)
 
 (* trigonometric transforms *)
 open Util
@@ -58,7 +59,8 @@ let interleave_zero input = fun i ->
     input ((i - 1) / 2)
 
 let trigII n input =
-  let fourn = 4 * n in
+  let twon = 2 * n 
+  and fourn = 4 * n in
   let input' = Complex.hermitian fourn (interleave_zero input)
   in
   Fft.dft 1 fourn input'
