@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-8 Matteo Frigo
- * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,44 +22,23 @@
 
 #include "ifftw.h"
 
-#define MAXNBUF ((INT)256)
-
-/* approx. 512KB of buffers for complex data */
-#define MAXBUFSZ (256 * 1024 / (INT)(sizeof(R)))
-
-INT X(nbuf)(INT n, INT vl)
+int X(compute_nbuf)(int n, int vl, int nbuf, int maxbufsz)
 {
-     INT i, nbuf, lb; 
+     int i; 
 
-     nbuf = X(imin)(MAXNBUF,
-		    X(imin)(vl, X(imax)((INT)1, MAXBUFSZ / n)));
+     if (nbuf * n > maxbufsz)
+          nbuf = X(imax)((int)1, maxbufsz / n);
 
      /*
-      * Look for a buffer number (not too small) that divides the
+      * Look for a buffer number (not too big) that divides the
       * vector length, in order that we only need one child plan:
       */
-     lb = X(imax)(1, nbuf / 4);
-     for (i = nbuf; i >= lb; --i)
+     for (i = nbuf; i < vl && i < 2 * nbuf; ++i)
           if (vl % i == 0)
                return i;
 
      /* whatever... */
+     nbuf = X(imin)(nbuf, vl);
      return nbuf;
 }
 
-#define SKEW 6 /* need to be even for SIMD */
-#define SKEWMOD 8 
-
-INT X(bufdist)(INT n, INT vl)
-{
-     if (vl == 1)
-	  return n;
-     else 
-	  /* return smallest X such that X >= N and X == SKEW (mod SKEWMOD) */
-	  return n + X(modulo)(SKEW - n, SKEWMOD);
-}
-
-int X(toobig)(INT n)
-{
-     return n > MAXBUFSZ;
-}
