@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2006 Matteo Frigo
- * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -23,7 +23,7 @@
  *
  */
 
-/* $Id: cycle.h,v 1.54 2006-02-15 13:43:05 athena Exp $ */
+/* $Id: cycle.h,v 1.35 2003/05/13 20:58:07 stevenj Exp $ */
 
 /* machine-dependent cycle counters code. Needs to be inlined. */
 
@@ -58,13 +58,13 @@
 
    AC_C_INLINE
    AC_HEADER_TIME
-   AC_CHECK_HEADERS([sys/time.h c_asm.h intrinsics.h mach/mach_time.h])
+   AC_CHECK_HEADERS([sys/time.h c_asm.h intrinsics.h])
 
    AC_CHECK_TYPE([hrtime_t],[AC_DEFINE(HAVE_HRTIME_T, 1, [Define to 1 if hrtime_t is defined in <sys/time.h>])],,[#if HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif])
 
-   AC_CHECK_FUNCS([gethrtime read_real_time time_base_to_time clock_gettime mach_absolute_time])
+   AC_CHECK_FUNCS([gethrtime read_real_time time_base_to_time clock_gettime])
 
    dnl Cray UNICOS _rtc() (real-time clock) intrinsic
    AC_MSG_CHECKING([for _rtc intrinsic])
@@ -153,17 +153,6 @@ INLINE_ELAPSED(__inline__)
 
 #define HAVE_TICK_COUNTER
 #endif
-
-/* MacOS/Mach (Darwin) time-base register interface (unlike UpTime,
-   from Carbon, requires no additional libraries to be linked). */
-#if defined(HAVE_MACH_ABSOLUTE_TIME) && defined(HAVE_MACH_MACH_TIME_H) && !defined(HAVE_TICK_COUNTER)
-#include <mach/mach_time.h>
-typedef uint64_t ticks;
-#define getticks mach_absolute_time
-INLINE_ELAPSED(__inline__)
-#define HAVE_TICK_COUNTER
-#endif
-
 /*----------------------------------------------------------------*/
 /*
  * Pentium cycle counter 
@@ -183,7 +172,6 @@ static __inline__ ticks getticks(void)
 INLINE_ELAPSED(__inline__)
 
 #define HAVE_TICK_COUNTER
-#define TIME_MIN 5000.0   /* unreliable pentium IV cycle counter */
 #endif
 
 /* Visual C++ -- thanks to Morten Nissov for his help with this */
@@ -194,14 +182,14 @@ typedef LARGE_INTEGER ticks;
 
 static __inline ticks getticks(void)
 {
-     ticks retval;
+     ticks ret;
 
      __asm {
 	  RDTSC
-	  mov retval.HighPart, edx
-	  mov retval.LowPart, eax
+	  mov ret.HighPart, edx
+	  mov ret.LowPart, eax
      }
-     return retval;
+     return ret;
 }
 
 static __inline double elapsed(ticks t1, ticks t0)
@@ -210,14 +198,13 @@ static __inline double elapsed(ticks t1, ticks t0)
 }  
 
 #define HAVE_TICK_COUNTER
-#define TIME_MIN 5000.0   /* unreliable pentium IV cycle counter */
 #endif
 
 /*----------------------------------------------------------------*/
 /*
  * X86-64 cycle counter
  */
-#if (defined(__GNUC__) || defined(__ICC)) && defined(__x86_64__)  && !defined(HAVE_TICK_COUNTER)
+#if defined(__GNUC__) && defined(__x86_64__)  && !defined(HAVE_TICK_COUNTER)
 typedef unsigned long long ticks;
 
 static __inline__ ticks getticks(void)
@@ -232,26 +219,12 @@ INLINE_ELAPSED(__inline__)
 #define HAVE_TICK_COUNTER
 #endif
 
-/* PGI compiler, courtesy Cristiano Calonaci, Andrea Tarsi, & Roberto Gori.
-   NOTE: this code will fail to link unless you use the -Masmkeyword compiler
-   option (grrr). */
-#if defined(__PGI) && defined(__x86_64__) && !defined(HAVE_TICK_COUNTER) 
-typedef unsigned long long ticks;
-static ticks getticks(void)
-{
-    asm(" rdtsc; shl    $0x20,%rdx; mov    %eax,%eax; or     %rdx,%rax;    ");
-}
-INLINE_ELAPSED(__inline__)
-#define HAVE_TICK_COUNTER
-#endif
+/* Visual C++ (FIXME: how to detect compilation for x86-64?) */
+#if _MSC_VER >= 1400 && !defined(HAVE_TICK_COUNTER)
+typedef ULONG64 ticks;
 
-/* Visual C++, courtesy of Dirk Michaelis */
-#if _MSC_VER >= 1400 && (defined(_M_AMD64) || defined(_M_X64)) && !defined(HAVE_TICK_COUNTER)
-
-#include <intrin.h>
-#pragma intrinsic(__rdtsc)
-typedef unsigned __int64 ticks;
 #define getticks __rdtsc
+
 INLINE_ELAPSED(__inline)
 
 #define HAVE_TICK_COUNTER
@@ -261,23 +234,6 @@ INLINE_ELAPSED(__inline)
 /*
  * IA64 cycle counter
  */
-
-/* intel's icc/ecc compiler */
-#if (defined(__EDG_VERSION) || defined(__ECC)) && defined(__ia64__) && !defined(HAVE_TICK_COUNTER)
-typedef unsigned long ticks;
-#include <ia64intrin.h>
-
-static __inline__ ticks getticks(void)
-{
-     return __getReg(_IA64_REG_AR_ITC);
-}
- 
-INLINE_ELAPSED(__inline__)
- 
-#define HAVE_TICK_COUNTER
-#endif
-
-/* gcc */
 #if defined(__GNUC__) && defined(__ia64__) && !defined(HAVE_TICK_COUNTER)
 typedef unsigned long ticks;
 
@@ -312,23 +268,18 @@ INLINE_ELAPSED(inline)
 #define HAVE_TICK_COUNTER
 #endif
 
-/* Microsoft Visual C++ */
-#if defined(_MSC_VER) && defined(_M_IA64) && !defined(HAVE_TICK_COUNTER)
-typedef unsigned __int64 ticks;
+/* intel's ecc compiler */
+#if defined(__ECC) && defined(__ia64__) && !defined(HAVE_TICK_COUNTER)
+typedef unsigned long ticks;
+#include <ia64intrin.h>
 
-#  ifdef __cplusplus
-extern "C"
-#  endif
-ticks __getReg(int whichReg);
-#pragma intrinsic(__getReg)
-
-static __inline ticks getticks(void)
+static __inline__ ticks getticks(void)
 {
-     volatile ticks temp;
-     temp = __getReg(3116);
-     return temp;
+     return __getReg(_IA64_REG_AR_ITC);
 }
-
+ 
+INLINE_ELAPSED(__inline__)
+ 
 #define HAVE_TICK_COUNTER
 #endif
 
@@ -406,7 +357,7 @@ typedef unsigned long ticks;
 static __inline__ ticks getticks(void)
 {
      ticks ret;
-     __asm__ __volatile__("rd %%tick, %0" : "=r" (ret));
+     __asm__("rd %%tick, %0" : "=r" (ret));
      return ret;
 }
 
@@ -416,7 +367,7 @@ INLINE_ELAPSED(__inline__)
 #endif
 
 /*----------------------------------------------------------------*/
-#if (defined(__DECC) || defined(__DECCXX)) && defined(__alpha) && defined(HAVE_C_ASM_H) && !defined(HAVE_TICK_COUNTER)
+#if defined(__DECC) && defined(__alpha) && defined(HAVE_C_ASM_H) && !defined(HAVE_TICK_COUNTER)
 #  include <c_asm.h>
 typedef unsigned int ticks;
 
