@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-8 Matteo Frigo
- * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  *
  */
 
+/* $Id: ifftw.h,v 1.226.2.2 2003/07/05 17:29:56 stevenj Exp $ */
 
 /* FFTW internal header file */
 #ifndef __IFFTW_H__
@@ -41,53 +42,35 @@
 # include <inttypes.h>           /* uintptr_t, maybe */
 #endif
 
-/* Windows annoyances -- since tests/hook.c uses some internal
-   FFTW functions, we need to given them the dllexport attribute
-   under Windows when compiling as a DLL (see api/fftw3.h). */
-#if defined(FFTW_EXTERN)
-#  define IFFTW_EXTERN FFTW_EXTERN
-#elif (defined(FFTW_DLL) || defined(DLL_EXPORT)) \
- && (defined(_WIN32) || defined(__WIN32__))
-#  define IFFTW_EXTERN extern __declspec(dllexport)
-#else
-#  define IFFTW_EXTERN extern
-#endif
-
 /* determine precision and name-mangling scheme */
 #define CONCAT(prefix, name) prefix ## name
 #if defined(FFTW_SINGLE)
-  typedef float R;
-# define X(name) CONCAT(fftwf_, name)
+typedef float R;
+#define X(name) CONCAT(fftwf_, name)
 #elif defined(FFTW_LDOUBLE)
-  typedef long double R;
-# define X(name) CONCAT(fftwl_, name)
-# define TRIGREAL_IS_LONG_DOUBLE
+typedef long double R;
+#define X(name) CONCAT(fftwl_, name)
 #else
-  typedef double R;
-# define X(name) CONCAT(fftw_, name)
+typedef double R;
+#define X(name) CONCAT(fftw_, name)
 #endif
 
-/*
-  integral type large enough to contain a stride (what ``int'' should
-  have been in the first place.
-*/
-typedef ptrdiff_t INT;
-
 /* dummy use of unused parameters to silence compiler warnings */
 #define UNUSED(x) (void)x
 
-#define NELEM(array) ((int) (sizeof(array) / sizeof((array)[0])))
-
 #define FFT_SIGN (-1)  /* sign convention for forward transforms */
-extern void X(extract_reim)(int sign, R *c, R **r, R **i);
 
+/* get rid of that object-oriented stink: */
 #define REGISTER_SOLVER(p, s) X(solver_register)(p, s)
 
 #define STRINGIZEx(x) #x
 #define STRINGIZE(x) STRINGIZEx(x)
 
-#if defined(HAVE_SSE) || defined(HAVE_SSE2) || defined(HAVE_ALTIVEC) || \
-    defined(HAVE_MIPS_PS)
+#ifndef HAVE_K7
+#define HAVE_K7 0
+#endif
+
+#if defined(HAVE_SSE) || defined(HAVE_SSE2) || defined(HAVE_ALTIVEC) || defined(HAVE_3DNOW)
 #define HAVE_SIMD 1
 #else
 #define HAVE_SIMD 0
@@ -103,7 +86,7 @@ typedef struct scanner_s scanner;
 
 /*-----------------------------------------------------------------------*/
 /* alloca: */
-#if HAVE_SIMD || HAVE_CELL
+#if HAVE_SIMD
 #define MIN_ALIGNMENT 16
 #endif
 
@@ -169,20 +152,9 @@ void *alloca(size_t);
 #  endif
 #endif
 
-/*-----------------------------------------------------------------------*/
-/* We can do an optimization for copying pairs of (aligned) floats
-   when in single precision if 2*float = double. */
-
-#define FFTW_2R_IS_DOUBLE (defined(FFTW_SINGLE) \
-                           && SIZEOF_FLOAT != 0 \
-                           && SIZEOF_DOUBLE == 2*SIZEOF_FLOAT)
-
-#define DOUBLE_ALIGNED(p) ((((uintptr_t)(p)) % sizeof(double)) == 0)
-
 /*-----------------------------------------------------------------------*/
 /* assert.c: */
-IFFTW_EXTERN void X(assertion_failed)(const char *s, 
-				      int line, const char *file);
+extern void X(assertion_failed)(const char *s, int line, const char *file);
 
 /* always check */
 #define CK(ex)						 \
@@ -199,11 +171,6 @@ IFFTW_EXTERN void X(assertion_failed)(const char *s,
 extern void X(debug)(const char *format, ...);
 #define D X(debug)
 
-/*-----------------------------------------------------------------------*/
-/* kalloc.c: */
-extern void *X(kernel_malloc)(size_t n);
-extern void X(kernel_free)(void *p);
-
 /*-----------------------------------------------------------------------*/
 /* alloc.c: */
 
@@ -224,26 +191,26 @@ enum malloc_tag {
      MALLOC_WHAT_LAST		/* must be last */
 };
 
-IFFTW_EXTERN void X(ifree)(void *ptr);
+extern void X(ifree)(void *ptr);
 extern void X(ifree0)(void *ptr);
 
 #ifdef FFTW_DEBUG_MALLOC
 
-IFFTW_EXTERN void *X(malloc_debug)(size_t n, enum malloc_tag what,
+extern void *X(malloc_debug)(size_t n, enum malloc_tag what,
 			     const char *file, int line);
 #define MALLOC(n, what) X(malloc_debug)(n, what, __FILE__, __LINE__)
 #define NATIVE_MALLOC(n, what) MALLOC(n, what)
-IFFTW_EXTERN void X(malloc_print_minfo)(int vrbose);
+void X(malloc_print_minfo)(int vrbose);
 
 #else /* ! FFTW_DEBUG_MALLOC */
 
-IFFTW_EXTERN void *X(malloc_plain)(size_t sz);
+extern void *X(malloc_plain)(size_t sz);
 #define MALLOC(n, what)  X(malloc_plain)(n)
 #define NATIVE_MALLOC(n, what) malloc(n)
 
 #endif
 
-#if defined(FFTW_DEBUG) && defined(FFTW_DEBUG_MALLOC) && (defined(HAVE_THREADS) || defined(HAVE_OPENMP))
+#if defined(FFTW_DEBUG) && defined(FFTW_DEBUG_MALLOC) && defined(HAVE_THREADS)
 extern int X(in_thread);
 #  define IN_THREAD X(in_thread)
 #  define THREAD_ON { int in_thread_save = X(in_thread); X(in_thread) = 1
@@ -254,41 +221,6 @@ extern int X(in_thread);
 #  define THREAD_OFF 
 #endif
 
-/*-----------------------------------------------------------------------*/
-/* low-resolution clock */
-
-#ifdef FAKE_CRUDE_TIME
- typedef int crude_time;
-#else
-# if TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-# else
-#  if HAVE_SYS_TIME_H
-#   include <sys/time.h>
-#  else
-#   include <time.h>
-#  endif
-# endif
-
-# ifdef HAVE_BSDGETTIMEOFDAY
-# ifndef HAVE_GETTIMEOFDAY
-# define gettimeofday BSDgettimeofday
-# define HAVE_GETTIMEOFDAY 1
-# endif
-# endif
-
-# if defined(HAVE_GETTIMEOFDAY)
-   typedef struct timeval crude_time;
-# else
-   typedef clock_t crude_time;
-# endif
-#endif /* else FAKE_CRUDE_TIME */
-
-crude_time X(get_crude_time)(void);
-double X(elapsed_since)(const planner *plnr, const problem *p,
-			crude_time t0); /* time in seconds since t0 */
-
 /*-----------------------------------------------------------------------*/
 /* ops.c: */
 /*
@@ -304,30 +236,27 @@ typedef struct {
 } opcnt;
 
 void X(ops_zero)(opcnt *dst);
-void X(ops_other)(INT o, opcnt *dst);
+void X(ops_other)(int o, opcnt *dst);
 void X(ops_cpy)(const opcnt *src, opcnt *dst);
 
 void X(ops_add)(const opcnt *a, const opcnt *b, opcnt *dst);
 void X(ops_add2)(const opcnt *a, opcnt *dst);
 
 /* dst = m * a + b */
-void X(ops_madd)(INT m, const opcnt *a, const opcnt *b, opcnt *dst);
+void X(ops_madd)(int m, const opcnt *a, const opcnt *b, opcnt *dst);
 
 /* dst += m * a */
-void X(ops_madd2)(INT m, const opcnt *a, opcnt *dst);
+void X(ops_madd2)(int m, const opcnt *a, opcnt *dst);
 
 
 /*-----------------------------------------------------------------------*/
 /* minmax.c: */
-INT X(imax)(INT a, INT b);
-INT X(imin)(INT a, INT b);
+int X(imax)(int a, int b);
+int X(imin)(int a, int b);
 
 /*-----------------------------------------------------------------------*/
 /* iabs.c: */
-INT X(iabs)(INT a);
-
-/* inline version */
-#define IABS(x) (((x) < 0) ? (0 - (x)) : (x))
+int X(iabs)(int a);
 
 /*-----------------------------------------------------------------------*/
 /* md5.c */
@@ -350,12 +279,12 @@ typedef struct {
 } md5;
 
 void X(md5begin)(md5 *p);
-void X(md5putb)(md5 *p, const void *d_, size_t len);
+void X(md5putb)(md5 *p, const void *d_, int len);
 void X(md5puts)(md5 *p, const char *s);
 void X(md5putc)(md5 *p, unsigned char c);
 void X(md5int)(md5 *p, int i);
-void X(md5INT)(md5 *p, INT i);
 void X(md5unsigned)(md5 *p, unsigned i);
+void X(md5ptrdiff)(md5 *p, ptrdiff_t d);
 void X(md5end)(md5 *p);
 
 /*-----------------------------------------------------------------------*/
@@ -364,9 +293,9 @@ void X(md5end)(md5 *p);
 #undef STRUCT_HACK_C99
 
 typedef struct {
-     INT n;
-     INT is;			/* input stride */
-     INT os;			/* output stride */
+     int n;
+     int is;			/* input stride */
+     int os;			/* output stride */
 } iodim;
 
 typedef struct {
@@ -394,31 +323,17 @@ typedef enum { INPLACE_IS, INPLACE_OS } inplace_kind;
 
 tensor *X(mktensor)(int rnk);
 tensor *X(mktensor_0d)(void);
-tensor *X(mktensor_1d)(INT n, INT is, INT os);
-tensor *X(mktensor_2d)(INT n0, INT is0, INT os0,
-		       INT n1, INT is1, INT os1);
-tensor *X(mktensor_3d)(INT n0, INT is0, INT os0,
-		       INT n1, INT is1, INT os1,
-		       INT n2, INT is2, INT os2);
-tensor *X(mktensor_4d)(INT n0, INT is0, INT os0,
-		       INT n1, INT is1, INT os1,
-		       INT n2, INT is2, INT os2,
-		       INT n3, INT is3, INT os3);
-tensor *X(mktensor_5d)(INT n0, INT is0, INT os0,
-		       INT n1, INT is1, INT os1,
-		       INT n2, INT is2, INT os2,
-		       INT n3, INT is3, INT os3,
-		       INT n4, INT is4, INT os4);
-INT X(tensor_sz)(const tensor *sz);
+tensor *X(mktensor_1d)(int n, int is, int os);
+tensor *X(mktensor_2d)(int n0, int is0, int os0,
+                      int n1, int is1, int os1);
+int X(tensor_sz)(const tensor *sz);
 void X(tensor_md5)(md5 *p, const tensor *t);
-INT X(tensor_max_index)(const tensor *sz);
-INT X(tensor_min_istride)(const tensor *sz);
-INT X(tensor_min_ostride)(const tensor *sz);
-INT X(tensor_min_stride)(const tensor *sz);
+int X(tensor_max_index)(const tensor *sz);
+int X(tensor_min_istride)(const tensor *sz);
+int X(tensor_min_ostride)(const tensor *sz);
+int X(tensor_min_stride)(const tensor *sz);
 int X(tensor_inplace_strides)(const tensor *sz);
 int X(tensor_inplace_strides2)(const tensor *a, const tensor *b);
-int X(tensor_strides_decrease)(const tensor *sz, const tensor *vecsz,
-                               inplace_kind k);
 tensor *X(tensor_copy)(const tensor *sz);
 int X(tensor_kosherp)(const tensor *x);
 
@@ -429,39 +344,19 @@ tensor *X(tensor_compress)(const tensor *sz);
 tensor *X(tensor_compress_contiguous)(const tensor *sz);
 tensor *X(tensor_append)(const tensor *a, const tensor *b);
 void X(tensor_split)(const tensor *sz, tensor **a, int a_rnk, tensor **b);
-int X(tensor_tornk1)(const tensor *t, INT *n, INT *is, INT *os);
+int X(tensor_tornk1)(const tensor *t, int *n, int *is, int *os);
 void X(tensor_destroy)(tensor *sz);
 void X(tensor_destroy2)(tensor *a, tensor *b);
 void X(tensor_destroy4)(tensor *a, tensor *b, tensor *c, tensor *d);
 void X(tensor_print)(const tensor *sz, printer *p);
 int X(dimcmp)(const iodim *a, const iodim *b);
-int X(tensor_equal)(const tensor *a, const tensor *b);
-int X(tensor_inplace_locations)(const tensor *sz, const tensor *vecsz);
 
 /*-----------------------------------------------------------------------*/
 /* problem.c: */
-enum { 
-     /* a problem that cannot be solved */
-     PROBLEM_UNSOLVABLE,
-
-     PROBLEM_DFT, 
-     PROBLEM_RDFT,
-     PROBLEM_RDFT2,
-
-     /* for mpi/ subdirectory */
-     PROBLEM_MPI_DFT,
-     PROBLEM_MPI_RDFT,
-     PROBLEM_MPI_RDFT2,
-     PROBLEM_MPI_TRANSPOSE,
-
-     PROBLEM_LAST 
-};
-
 typedef struct {
-     int problem_kind;
      void (*hash) (const problem *ego, md5 *p);
      void (*zero) (const problem *ego);
-     void (*print) (const problem *ego, printer *p);
+     void (*print) (problem *ego, printer *p);
      void (*destroy) (problem *ego);
 } problem_adt;
 
@@ -471,7 +366,6 @@ struct problem_s {
 
 problem *X(mkproblem)(size_t sz, const problem_adt *adt);
 void X(problem_destroy)(problem *ego);
-problem *X(mkproblem_unsolvable)(void);
 
 /*-----------------------------------------------------------------------*/
 /* print.c */
@@ -487,7 +381,7 @@ struct printer_s {
 printer *X(mkprinter)(size_t size, 
 		      void (*putchr)(printer *p, char c),
 		      void (*cleanup)(printer *p));
-IFFTW_EXTERN void X(printer_destroy)(printer *p);
+void X(printer_destroy)(printer *p);
 
 /*-----------------------------------------------------------------------*/
 /* scan.c */
@@ -503,40 +397,30 @@ void X(scanner_destroy)(scanner *sc);
 
 /*-----------------------------------------------------------------------*/
 /* plan.c: */
-
-enum wakefulness {
-     SLEEPY,
-     AWAKE_ZERO,
-     AWAKE_SQRTN_TABLE,
-     AWAKE_SINCOS
-};
-
 typedef struct {
      void (*solve)(const plan *ego, const problem *p);
-     void (*awake)(plan *ego, enum wakefulness wakefulness);
+     void (*awake)(plan *ego, int flag);
      void (*print)(const plan *ego, printer *p);
      void (*destroy)(plan *ego);
 } plan_adt;
 
 struct plan_s {
      const plan_adt *adt;
+     int awake_refcnt;
      opcnt ops;
      double pcost;
-     enum wakefulness wakefulness; /* used for debugging only */
-     int could_prune_now_p;
 };
 
 plan *X(mkplan)(size_t size, const plan_adt *adt);
 void X(plan_destroy_internal)(plan *ego);
-IFFTW_EXTERN void X(plan_awake)(plan *ego, enum wakefulness wakefulness);
+void X(plan_awake)(plan *ego, int flag);
+#define AWAKE(plan, flag) X(plan_awake)(plan, flag)
 void X(plan_null_destroy)(plan *ego);
 
 /*-----------------------------------------------------------------------*/
 /* solver.c: */
 typedef struct {
-     int problem_kind;
      plan *(*mkplan)(const solver *ego, const problem *p, planner *plnr);
-     void (*destroy)(solver *ego);
 } solver_adt;
 
 struct solver_s {
@@ -560,169 +444,111 @@ typedef struct slvdesc_s {
      const char *reg_nam;
      unsigned nam_hash;
      int reg_id;
-     int next_for_same_problem_kind;
 } slvdesc;
 
 typedef struct solution_s solution; /* opaque */
 
-/* interpretation of L and U: 
+/* values for problem_flags: */
+enum { 
+     DESTROY_INPUT = 0x1,
+     NO_SIMD = 0x2,
+     CONSERVE_MEMORY = 0x4,
+     NO_DHT_R2HC = 0x8
+};
 
-   - if it returns a plan, the planner guarantees that all applicable
-     plans at least as impatient as U have been tried, and that each
-     plan in the solution is at least as impatient as L.
-   
-   - if it returns 0, the planner guarantees to have tried all solvers
-     at least as impatient as L, and that none of them was applicable.
+#define DESTROY_INPUTP(plnr) ((plnr)->problem_flags & DESTROY_INPUT)
+#define NO_SIMDP(plnr) ((plnr)->problem_flags & NO_SIMD)
+#define CONSERVE_MEMORYP(plnr) ((plnr)->problem_flags & CONSERVE_MEMORY)
+#define NO_DHT_R2HCP(plnr) ((plnr)->problem_flags & NO_DHT_R2HC)
 
-   The structure is packed to fit into 64 bits.
-*/
+/* values for planner_flags: */
+enum {
+     /* impatience flags  */
 
-typedef struct {
-     unsigned l:20;
-     unsigned hash_info:3;
-#    define BITS_FOR_TIMELIMIT 9
-     unsigned timelimit_impatience:BITS_FOR_TIMELIMIT;
-     unsigned u:20;
+     BELIEVE_PCOST = 0x1,
+     DFT_R2HC_ICKY = 0x2,
+     NONTHREADED_ICKY = 0x4,
+     NO_BUFFERING = 0x8,
+     NO_EXHAUSTIVE = 0x10,
+     NO_INDIRECT_OP = 0x20,
+     NO_LARGE_GENERIC = 0x40,
+     NO_RANK_SPLITS = 0x80,
+     NO_VRANK_SPLITS = 0x100,
+     NO_VRECURSE = 0x200,
      
-     /* abstraction break: we store the solver here to pad the
-	structure to 64 bits.  Otherwise, the struct is padded to 64
-	bits anyway, and another word is allocated for slvndx. */
-#    define BITS_FOR_SLVNDX 12
-     unsigned slvndx:BITS_FOR_SLVNDX;
-} flags_t;
+     /* flags that control the search */
+     NO_UGLY = 0x400,  /* avoid plans we are 99% sure are suboptimal */
+     NO_SEARCH = 0x800,  /* avoid searching altogether---use wisdom entries 
+			    only */
 
-/* impatience flags  */
-enum {
-     BELIEVE_PCOST = 0x0001,
-     ESTIMATE = 0x0002,
-     NO_DFT_R2HC = 0x0004,
-     NO_SLOW = 0x0008,
-     NO_VRECURSE = 0x0010,
-     NO_INDIRECT_OP = 0x0020,
-     NO_LARGE_GENERIC = 0x0040,
-     NO_RANK_SPLITS = 0x0080,
-     NO_VRANK_SPLITS = 0x0100,
-     NO_NONTHREADED = 0x0200,
-     NO_BUFFERING = 0x0400,
-     NO_FIXED_RADIX_LARGE_N = 0x0800,
-     NO_DESTROY_INPUT = 0x1000,
-     NO_SIMD = 0x2000,
-     CONSERVE_MEMORY = 0x4000,
-     NO_DHT_R2HC = 0x8000,
-     NO_UGLY = 0x10000,
-     ALLOW_PRUNING = 0x20000
+     ESTIMATE = 0x1000,
+     IMPATIENCE_FLAGS = (ESTIMATE | (ESTIMATE - 1)),
+     
+     BLESSING = 0x4000,  /* save this entry */
+     H_VALID = 0x8000,    /* valid hastable entry */
+     NONIMPATIENCE_FLAGS = BLESSING
 };
 
-/* hashtable information */
-enum {
-     BLESSING = 0x1,   /* save this entry */
-     H_VALID = 0x2,    /* valid hastable entry */
-     H_LIVE = 0x4      /* entry is nonempty, implies H_VALID */
-};
-
-#define PLNR_L(plnr) ((plnr)->flags.l)
-#define PLNR_U(plnr) ((plnr)->flags.u)
-#define PLNR_TIMELIMIT_IMPATIENCE(plnr) ((plnr)->flags.timelimit_impatience)
-
-#define ESTIMATEP(plnr) (PLNR_U(plnr) & ESTIMATE)
-#define BELIEVE_PCOSTP(plnr) (PLNR_U(plnr) & BELIEVE_PCOST)
-#define ALLOW_PRUNINGP(plnr) (PLNR_U(plnr) & ALLOW_PRUNING)
-
-#define NO_INDIRECT_OP_P(plnr) (PLNR_L(plnr) & NO_INDIRECT_OP)
-#define NO_LARGE_GENERICP(plnr) (PLNR_L(plnr) & NO_LARGE_GENERIC)
-#define NO_RANK_SPLITSP(plnr) (PLNR_L(plnr) & NO_RANK_SPLITS)
-#define NO_VRANK_SPLITSP(plnr) (PLNR_L(plnr) & NO_VRANK_SPLITS)
-#define NO_VRECURSEP(plnr) (PLNR_L(plnr) & NO_VRECURSE)
-#define NO_DFT_R2HCP(plnr) (PLNR_L(plnr) & NO_DFT_R2HC)
-#define NO_SLOWP(plnr) (PLNR_L(plnr) & NO_SLOW)
-#define NO_UGLYP(plnr) (PLNR_L(plnr) & NO_UGLY)
-#define NO_FIXED_RADIX_LARGE_NP(plnr) \
-  (PLNR_L(plnr) & NO_FIXED_RADIX_LARGE_N)
-#define NO_NONTHREADEDP(plnr) \
-  ((PLNR_L(plnr) & NO_NONTHREADED) && (plnr)->nthr > 1)
-
-#define NO_DESTROY_INPUTP(plnr) (PLNR_L(plnr) & NO_DESTROY_INPUT)
-#define NO_SIMDP(plnr) (PLNR_L(plnr) & NO_SIMD)
-#define CONSERVE_MEMORYP(plnr) (PLNR_L(plnr) & CONSERVE_MEMORY)
-#define NO_DHT_R2HCP(plnr) (PLNR_L(plnr) & NO_DHT_R2HC)
-#define NO_BUFFERINGP(plnr) (PLNR_L(plnr) & NO_BUFFERING)
+#define BELIEVE_PCOSTP(plnr) ((plnr)->planner_flags & BELIEVE_PCOST)
+#define DFT_R2HC_ICKYP(plnr) ((plnr)->planner_flags & DFT_R2HC_ICKY)
+#define ESTIMATEP(plnr) ((plnr)->planner_flags & ESTIMATE)
+#define NONTHREADED_ICKYP(plnr) (((plnr)->planner_flags & NONTHREADED_ICKY) \
+                                  && (plnr)->nthr > 1)
+#define NO_BUFFERINGP(plnr) ((plnr)->planner_flags & NO_BUFFERING)
+#define NO_EXHAUSTIVEP(plnr) ((plnr)->planner_flags & NO_EXHAUSTIVE)
+#define NO_INDIRECT_OP_P(plnr) ((plnr)->planner_flags & NO_INDIRECT_OP)
+#define NO_LARGE_GENERICP(plnr) ((plnr)->planner_flags & NO_LARGE_GENERIC)
+#define NO_RANK_SPLITSP(plnr) ((plnr)->planner_flags & NO_RANK_SPLITS)
+#define NO_UGLYP(plnr) ((plnr)->planner_flags & NO_UGLY)
+#define NO_SEARCHP(plnr) ((plnr)->planner_flags & NO_SEARCH)
+#define NO_VRANK_SPLITSP(plnr) ((plnr)->planner_flags & NO_VRANK_SPLITS)
+#define NO_VRECURSEP(plnr) ((plnr)->planner_flags & NO_VRECURSE)
 
 typedef enum { FORGET_ACCURSED, FORGET_EVERYTHING } amnesia;
 
-typedef enum { 
-     /* WISDOM_NORMAL: planner may or may not use wisdom */
-     WISDOM_NORMAL, 
-
-     /* WISDOM_ONLY: planner must use wisdom and must avoid searching */
-     WISDOM_ONLY, 
-
-     /* WISDOM_IS_BOGUS: planner must return 0 as quickly as possible */
-     WISDOM_IS_BOGUS,
-
-     /* WISDOM_IGNORE_INFEASIBLE: planner ignores infeasible wisdom */
-     WISDOM_IGNORE_INFEASIBLE,
-
-     /* WISDOM_IGNORE_ALL: planner ignores all */
-     WISDOM_IGNORE_ALL
-} wisdom_state_t;
-
 typedef struct {
      void (*register_solver)(planner *ego, solver *s);
-     plan *(*mkplan)(planner *ego, const problem *p);
+     plan *(*mkplan)(planner *ego, problem *p);
      void (*forget)(planner *ego, amnesia a);
      void (*exprt)(planner *ego, printer *p); /* ``export'' is a reserved
 						 word in C++. */
      int (*imprt)(planner *ego, scanner *sc);
 } planner_adt;
 
-/* hash table of solutions */
-typedef struct {
-     solution *solutions;
-     unsigned hashsiz, nelem;
-
-     /* statistics */
-     int lookup, succ_lookup, lookup_iter;
-     int insert, insert_iter, insert_unknown;
-     int nrehash;
-} hashtab;
-
-typedef enum { COST_SUM, COST_MAX } cost_kind;
-
 struct planner_s {
      const planner_adt *adt;
-     void (*hook)(struct planner_s *plnr, plan *pln, 
-		  const problem *p, int optimalp);
-     double (*cost_hook)(const problem *p, double t, cost_kind k);
+     void (*hook)(plan *pln, const problem *p, int optimalp);
 
      /* solver descriptors */
      slvdesc *slvdescs;
      unsigned nslvdesc, slvdescsiz;
      const char *cur_reg_nam;
      int cur_reg_id;
-     int slvdescs_for_problem_kind[PROBLEM_LAST];
 
-     wisdom_state_t wisdom_state;
-
-     hashtab htab_blessed;
-     hashtab htab_unblessed;
+     /* hash table of solutions */
+     solution *solutions;
+     unsigned hashsiz, nelem;
 
      int nthr;
-     flags_t flags;
-
-     crude_time start_time;
-     double timelimit; /* elapsed_since(start_time) at which to bail out */
-     int timed_out; /* whether most recent search timed out */
-     int need_timeout_check;
-
+     unsigned problem_flags;
+     unsigned short planner_flags; /* matches type of solution.flags in
+				      planner.c */
      /* various statistics */
      int nplan;    /* number of plans evaluated */
      double pcost, epcost; /* total pcost of measured/estimated plans */
      int nprob;    /* number of problems evaluated */
+     int lookup, succ_lookup, lookup_iter;
+     int insert, insert_iter, insert_unknown;
+     int nrehash;
 };
 
 planner *X(mkplanner)(void);
 void X(planner_destroy)(planner *ego);
 
+#ifdef FFTW_DEBUG
+void X(planner_dump)(planner *ego, int vrbose);
+#endif
+
 /*
   Iterate over all solvers.   Read:
  
@@ -745,44 +571,26 @@ void X(planner_destroy)(planner *ego);
      }							\
 }
 
-#define FORALL_SOLVERS_OF_KIND(kind, ego, s, p, what)		\
-{								\
-     int _cnt = ego->slvdescs_for_problem_kind[kind]; 		\
-     while (_cnt >= 0) {					\
-	  slvdesc *p = ego->slvdescs + _cnt;			\
-	  solver *s = p->slv;					\
-	  what;							\
-	  _cnt = p->next_for_same_problem_kind;			\
-     }								\
-}
-
-
 /* make plan, destroy problem */
 plan *X(mkplan_d)(planner *ego, problem *p);
-plan *X(mkplan_f_d)(planner *ego, problem *p, 
-		    unsigned l_set, unsigned u_set, unsigned u_reset);
 
 /*-----------------------------------------------------------------------*/
 /* stride.c: */
 
 /* If PRECOMPUTE_ARRAY_INDICES is defined, precompute all strides. */
-#if (defined(__i386__) || defined(__x86_64__) || _M_IX86 >= 500) && !defined(FFTW_LDOUBLE)
+#if (defined(__i386__) || _M_IX86 >= 500) && !HAVE_K7 && !defined(FFTW_LDOUBLE)
 #define PRECOMPUTE_ARRAY_INDICES
 #endif
 
-extern const INT X(an_INT_guaranteed_to_be_zero);
-
 #ifdef PRECOMPUTE_ARRAY_INDICES
-typedef INT *stride;
+typedef int *stride;
 #define WS(stride, i)  (stride[i])
-extern stride X(mkstride)(INT n, INT s);
+extern stride X(mkstride)(int n, int s);
 void X(stride_destroy)(stride p);
-/* hackery to prevent the compiler from copying the strides array
-   onto the stack */
-#define MAKE_VOLATILE_STRIDE(x) (x) = (x) + X(an_INT_guaranteed_to_be_zero)
+
 #else
 
-typedef INT stride;
+typedef int stride;
 #define WS(stride, i)  (stride * i)
 #define fftwf_mkstride(n, stride) stride
 #define fftw_mkstride(n, stride) stride
@@ -791,10 +599,6 @@ typedef INT stride;
 #define fftw_stride_destroy(p) ((void) p)
 #define fftwl_stride_destroy(p) ((void) p)
 
-/* hackery to prevent the compiler from ``optimizing'' induction
-   variables in codelet loops. */
-#define MAKE_VOLATILE_STRIDE(x) (x) = (x) ^ X(an_INT_guaranteed_to_be_zero)
-
 #endif /* PRECOMPUTE_ARRAY_INDICES */
 
 /*-----------------------------------------------------------------------*/
@@ -814,157 +618,103 @@ int X(pickdim)(int which_dim, const int *buddies, int nbuddies,
 /*-----------------------------------------------------------------------*/
 /* twiddle.c */
 /* little language to express twiddle factors computation */
-enum { TW_COS = 0, TW_SIN = 1, TW_CEXP = 2, TW_NEXT = 3, 
-       TW_FULL = 4, TW_HALF = 5 };
+enum { TW_COS = 0, TW_SIN = 1, TW_TAN = 2, TW_NEXT = 3,
+       TW_FULL = 4, TW_GENERIC = 5 };
 
 typedef struct {
      unsigned char op;
-     signed char v;
+     unsigned char v;
      short i;
 } tw_instr;
 
 typedef struct twid_s {
      R *W;                     /* array of twiddle factors */
-     INT n, r, m;                /* transform order, radix, # twiddle rows */
+     int n, r, m;                /* transform order, radix, # twiddle rows */
      int refcnt;
      const tw_instr *instr;
      struct twid_s *cdr;
-     enum wakefulness wakefulness;
 } twid;
 
-INT X(twiddle_length)(INT r, const tw_instr *p);
-void X(twiddle_awake)(enum wakefulness wakefulness,
-		      twid **pp, const tw_instr *instr, INT n, INT r, INT m);
+void X(mktwiddle)(twid **pp, const tw_instr *instr, int n, int r, int m);
+void X(twiddle_destroy)(twid **pp);
+int X(twiddle_length)(int r, const tw_instr *p);
+void X(twiddle_awake)(int flg, twid **pp, 
+		      const tw_instr *instr, int n, int r, int m);
 
 /*-----------------------------------------------------------------------*/
 /* trig.c */
-#ifdef TRIGREAL_IS_LONG_DOUBLE
-   typedef long double trigreal;
+#ifdef FFTW_LDOUBLE
+typedef long double trigreal;
 #else
-   typedef double trigreal;
+typedef double trigreal;
 #endif
 
-typedef struct triggen_s triggen;
-
-struct triggen_s {
-     void (*cexp)(triggen *t, INT m, R *result);
-     void (*cexpl)(triggen *t, INT m, trigreal *result);
-     void (*rotate)(triggen *p, INT m, R xr, R xi, R *res);
-
-     INT twshft;
-     INT twradix;
-     INT twmsk;
-     trigreal *W0, *W1;
-     INT n;
-};
-
-triggen *X(mktriggen)(enum wakefulness wakefulness, INT n);
-void X(triggen_destroy)(triggen *p);
+extern trigreal X(cos2pi)(int, int);
+extern trigreal X(sin2pi)(int, int);
+extern trigreal X(tan2pi)(int, int);
+extern trigreal X(sincos)(trigreal m, trigreal n, int sinp);
 
 /*-----------------------------------------------------------------------*/
 /* primes.c: */
 
-#define MULMOD(x, y, p) \
-   (((x) <= 92681 - (y)) ? ((x) * (y)) % (p) : X(safe_mulmod)(x, y, p))
+#if defined(FFTW_ENABLE_UNSAFE_MULMOD)
+#  define MULMOD(x,y,p) (((x) * (y)) % (p))
+#elif ((SIZEOF_INT != 0) && (SIZEOF_LONG >= 2 * SIZEOF_INT))
+#  define MULMOD(x,y,p) ((int) ((((long) (x)) * ((long) (y))) % ((long) (p))))
+#elif ((SIZEOF_INT != 0) && (SIZEOF_LONG_LONG >= 2 * SIZEOF_INT))
+#  define MULMOD(x,y,p) ((int) ((((long long) (x)) * ((long long) (y))) \
+				 % ((long long) (p))))
+#elif defined(_MSC_VER)
+#  define MULMOD(x,y,p) ((int) ((((__int64) (x)) * ((__int64) (y))) \
+                                 % ((__int64) (p))))
+#else /* 'long long' unavailable */
+#  define SAFE_MULMOD 1
+int X(safe_mulmod)(int x, int y, int p);
+#  define MULMOD(x,y,p) X(safe_mulmod)(x,y,p)
+#endif
 
-INT X(safe_mulmod)(INT x, INT y, INT p);
-INT X(power_mod)(INT n, INT m, INT p);
-INT X(find_generator)(INT p);
-INT X(first_divisor)(INT n);
-int X(is_prime)(INT n);
-INT X(next_prime)(INT n);
-int X(factors_into)(INT n, const INT *primes);
-INT X(choose_radix)(INT r, INT n);
-INT X(isqrt)(INT n);
-INT X(modulo)(INT a, INT n);
+int X(power_mod)(int n, int m, int p);
+int X(find_generator)(int p);
+int X(first_divisor)(int n);
+int X(is_prime)(int n);
+int X(next_prime)(int n);
 
-#define GENERIC_MIN_BAD 173 /* min prime for which generic becomes bad */
+#define GENERIC_MIN_BAD 71 /* min prime for which generic becomes bad */
 
 /*-----------------------------------------------------------------------*/
 /* rader.c: */
 typedef struct rader_tls rader_tl;
 
-void X(rader_tl_insert)(INT k1, INT k2, INT k3, R *W, rader_tl **tl);
-R *X(rader_tl_find)(INT k1, INT k2, INT k3, rader_tl *t);
+void X(rader_tl_insert)(int k1, int k2, int k3, R *W, rader_tl **tl);
+R *X(rader_tl_find)(int k1, int k2, int k3, rader_tl *t);
 void X(rader_tl_delete)(R *W, rader_tl **tl);
 
 /*-----------------------------------------------------------------------*/
-/* copy/transposition routines */
+/* transpose.c: */
 
-/* lower bound to the cache size, for tiled routines */
-#define CACHESIZE 8192
-
-INT X(compute_tilesz)(INT vl, int how_many_tiles_in_cache);
-
-void X(tile2d)(INT n0l, INT n0u, INT n1l, INT n1u, INT tilesz,
-	       void (*f)(INT n0l, INT n0u, INT n1l, INT n1u, void *args),
-	       void *args);
-void X(cpy1d)(R *I, R *O, INT n0, INT is0, INT os0, INT vl);
-void X(cpy2d)(R *I, R *O,
-	      INT n0, INT is0, INT os0,
-	      INT n1, INT is1, INT os1,
-	      INT vl);
-void X(cpy2d_ci)(R *I, R *O,
-		 INT n0, INT is0, INT os0,
-		 INT n1, INT is1, INT os1,
-		 INT vl);
-void X(cpy2d_co)(R *I, R *O,
-		 INT n0, INT is0, INT os0,
-		 INT n1, INT is1, INT os1,
-		 INT vl);
-void X(cpy2d_tiled)(R *I, R *O,
-		    INT n0, INT is0, INT os0,
-		    INT n1, INT is1, INT os1, 
-		    INT vl);
-void X(cpy2d_tiledbuf)(R *I, R *O,
-		       INT n0, INT is0, INT os0,
-		       INT n1, INT is1, INT os1, 
-		       INT vl);
-void X(cpy2d_pair)(R *I0, R *I1, R *O0, R *O1,
-		   INT n0, INT is0, INT os0,
-		   INT n1, INT is1, INT os1);
-void X(cpy2d_pair_ci)(R *I0, R *I1, R *O0, R *O1,
-		      INT n0, INT is0, INT os0,
-		      INT n1, INT is1, INT os1);
-void X(cpy2d_pair_co)(R *I0, R *I1, R *O0, R *O1,
-		      INT n0, INT is0, INT os0,
-		      INT n1, INT is1, INT os1);
-
-void X(transpose)(R *I, INT n, INT s0, INT s1, INT vl);
-void X(transpose_tiled)(R *I, INT n, INT s0, INT s1, INT vl);
-void X(transpose_tiledbuf)(R *I, INT n, INT s0, INT s1, INT vl);
-
-typedef void (*transpose_func)(R *I, INT n, INT s0, INT s1, INT vl);
-typedef void (*cpy2d_func)(R *I, R *O,
-			   INT n0, INT is0, INT os0,
-			   INT n1, INT is1, INT os1,
-			   INT vl);
-
-#if HAVE_CELL
-int X(cell_transpose_applicable)(R *I, const iodim *d, INT vl);
-void X(cell_transpose)(R *I, INT n, INT s0, INT s1, INT vl);
-int X(cell_copy_applicable)(R *I, R *O, const iodim *n, const iodim *v);
-void X(cell_copy)(R *I, R *O, const iodim *n, const iodim *v);
-#endif
+void X(transpose)(R *A, int n, int m, int d, int N, R *buf);
+void X(transpose_slow)(R *a, int nx, int ny, int N,
+		       char *move, int move_size, R *buf);
+int X(transposable)(const iodim *a, const iodim *b,
+		    int vl, int s, R *ri, R *ii);
+void X(transpose_dims)(const iodim *a, const iodim *b,
+                       int *n, int *m, int *d, int *nd, int *md);
+int X(transpose_simplep)(const iodim *a, const iodim *b, int vl, int s,
+			 R *ri, R *ii);
+int X(transpose_slowp)(const iodim *a, const iodim *b, int N);
 
 /*-----------------------------------------------------------------------*/
 /* misc stuff */
-void X(null_awake)(plan *ego, enum wakefulness wakefulness);
-double X(iestimate_cost)(const planner *, const plan *, const problem *);
-
-double X(measure_execution_time)(const planner *plnr, 
-				 plan *pln, const problem *p);
+void X(null_awake)(plan *ego, int awake);
+int X(square)(int x);
+double X(measure_execution_time)(plan *pln, const problem *p);
 int X(alignment_of)(R *p);
 unsigned X(hash)(const char *s);
-INT X(nbuf)(INT n, INT vl, INT maxnbuf);
-int X(nbuf_redundant)(INT n, INT vl, int which, 
-		      const INT *maxnbuf, int nmaxnbuf);
-INT X(bufdist)(INT n, INT vl);
-int X(toobig)(INT n);
-int X(ct_uglyp)(INT min_n, INT n, INT r);
+int X(compute_nbuf)(int n, int vl, int nbuf, int maxbufsz);
+int X(ct_uglyp)(int min_n, int n, int r);
 
-#if HAVE_SIMD || HAVE_CELL
-R *X(taint)(R *p, INT s);
+#if HAVE_SIMD
+R *X(taint)(R *p, int s);
 R *X(join_taint)(R *p1, R *p2);
 #define TAINT(p, s) X(taint)(p, s)
 #define UNTAINT(p) ((R *) (((uintptr_t) (p)) & ~(uintptr_t)3))
@@ -1002,48 +752,13 @@ typedef R E;  /* internal precision of codelets. */
 
 /* FMA macros */
 
-#if defined(__GNUC__) && (defined(__powerpc__) || defined(__ppc__) || defined(_POWER))
-/* The obvious expression a * b + c does not work.  If both x = a * b
+#if defined(__GNUC__) && (defined(__powerpc__) || defined(__ppc__))
+/* this peculiar coding seems to do the right thing on all of
+   gcc-2.95, gcc-3.1, and gcc-3.2.  
+
+   The obvious expression a * b + c does not work.  If both x = a * b
    + c and y = a * b - c appear in the source, gcc computes t = a * b,
    x = t + c, y = t - c, thus destroying the fma.
-
-   This peculiar coding seems to do the right thing on all of
-   gcc-2.95, gcc-3.1, gcc-3.2, and gcc-3.3.  It does the right thing
-   on gcc-3.4 -fno-web (because the ``web'' pass splits the variable
-   `x' for the single-assignment form).
-
-   However, gcc-4.0 is a formidable adversary which succeeds in
-   pessimizing two fma's into one multiplication and two additions.
-   It does it very early in the game---before the optimization passes
-   even start.  The only real workaround seems to use fake inline asm
-   such as
-
-     asm ("# confuse gcc %0" : "=f"(a) : "0"(a));
-     return a * b + c;
-     
-   in each of the FMA, FMS, FNMA, and FNMS functions.  However, this
-   does not solve the problem either, because two equal asm statements
-   count as a common subexpression!  One must use *different* fake asm
-   statements:
-
-   in FMA:
-     asm ("# confuse gcc for fma %0" : "=f"(a) : "0"(a));
-
-   in FMS:
-     asm ("# confuse gcc for fms %0" : "=f"(a) : "0"(a));
-
-   etc.
-
-   After these changes, gcc recalcitrantly generates the fma that was
-   in the source to begin with.  However, the extra asm() cruft
-   confuses other passes of gcc, notably the instruction scheduler.
-   (Of course, one could also generate the fma directly via inline
-   asm, but this confuses the scheduler even more.)
-
-   Steven and I have submitted more than one bug report to the gcc
-   mailing list over the past few years, to no effect.  Thus, I give
-   up.  gcc-4.0 can go to hell.  I'll wait at least until gcc-4.3 is
-   out before touching this crap again.
 */
 static __inline__ E FMA(E a, E b, E c)
 {
@@ -1079,21 +794,9 @@ static __inline__ E FNMS(E a, E b, E c)
 #define FNMS(a, b, c) ((c) - ((a) * (b)))
 #endif
 
-/* stack-alignment hackery */
-#ifdef __ICC /* Intel's compiler for ia32 */
-#define WITH_ALIGNED_STACK(what)				\
-{								\
-     /*								\
-      * Simply calling alloca seems to do the right thing.	\
-      * The size of the allocated block seems to be irrelevant.	\
-      */							\
-     _alloca(16);						\
-     what							\
-}
-#endif
 
-#if defined(__GNUC__) && defined(__i386__) && !defined(WITH_ALIGNED_STACK) \
-    && !(defined(__MACOSX__) || defined(__APPLE__)) /* OSX ABI is aligned */
+/* stack-alignment hackery */
+#if defined(__GNUC__) && defined(__i386__)
 /*
  * horrible hack to align the stack to a 16-byte boundary.
  *
@@ -1126,6 +829,18 @@ static __inline__ E FNMS(E a, E b, E c)
 }
 #endif
 
+#ifdef __ICC /* Intel's compiler for ia32 */
+#define WITH_ALIGNED_STACK(what)				\
+{								\
+     /*								\
+      * Simply calling alloca seems to do the right thing.	\
+      * The size of the allocated block seems to be irrelevant.	\
+      */							\
+     _alloca(16);						\
+     what							\
+}
+#endif
+
 #ifndef WITH_ALIGNED_STACK
 #define WITH_ALIGNED_STACK(what) what
 #endif
