@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,15 +18,16 @@
  *
  */
 
+/* $Id: minmax.c,v 1.6 2003/03/15 20:29:43 stevenj Exp $ */
 
 #include "ifftw.h"
 
-INT X(imax)(INT a, INT b)
+int X(imax)(int a, int b)
 {
      return (a > b) ? a : b;
 }
 
-INT X(imin)(INT a, INT b)
+int X(imin)(int a, int b)
 {
      return (a < b) ? a : b;
 }
