@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  *
  */
 
+/* $Id: solver.c,v 1.5 2003/03/15 20:29:43 stevenj Exp $ */
 
 #include "ifftw.h"
 
@@ -37,11 +38,8 @@ void X(solver_use)(solver *ego)
 
 void X(solver_destroy)(solver *ego)
 {
-     if ((--ego->refcnt) == 0) {
-	  if (ego->adt->destroy)
-	       ego->adt->destroy(ego);
+     if ((--ego->refcnt) == 0)
           X(ifree)(ego);
-     }
 }
 
 void X(solver_register)(planner *plnr, solver *s)
