@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,15 +18,14 @@
  *
  */
 
+/* $Id: stride.c,v 1.7 2003/03/15 20:29:43 stevenj Exp $ */
 #include "ifftw.h"
 
-const INT X(an_INT_guaranteed_to_be_zero) = 0;
-
 #ifdef PRECOMPUTE_ARRAY_INDICES
-stride X(mkstride)(INT n, INT s)
+stride X(mkstride)(int n, int s)
 {
      int i;
-     INT *p = (INT *) MALLOC(n * sizeof(INT), STRIDES);
+     int *p = (int *) MALLOC(n * sizeof(int), STRIDES);
 
      for (i = 0; i < n; ++i)
           p[i] = s * i;
