@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-8 Matteo Frigo
- * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  *
  */
 
+/* $Id: timer.c,v 1.16 2003/05/08 00:46:10 stevenj Exp $ */
 
 #include "ifftw.h"
 
@@ -27,64 +28,71 @@
 
 #ifndef WITH_SLOW_TIMER
 #  include "cycle.h"
+#else
+#  if TIME_WITH_SYS_TIME
+#   include <sys/time.h>
+#   include <time.h>
+#  else
+#   if HAVE_SYS_TIME_H
+#    include <sys/time.h>
+#   else
+#    include <time.h>
+#   endif
+#  endif
 #endif
 
 #ifndef FFTW_TIME_LIMIT
 #define FFTW_TIME_LIMIT 2.0  /* don't run for more than two seconds */
 #endif
 
-#if defined(HAVE_GETTIMEOFDAY)
-crude_time X(get_crude_time)(void)
+#ifdef HAVE_BSDGETTIMEOFDAY
+#ifndef HAVE_GETTIMEOFDAY
+#define gettimeofday BSDgettimeofday
+#define HAVE_GETTIMEOFDAY 1
+#endif
+#endif
+
+#if defined(HAVE_GETTIMEOFDAY) && !defined(HAVE_SECONDS_TIMER)
+typedef struct timeval seconds;
+
+static seconds getseconds(void)
 {
-     crude_time tv;
+     struct timeval tv;
      gettimeofday(&tv, 0);
      return tv;
 }
 
-#define elapsed_sec(t1,t0) ((double)(t1.tv_sec - t0.tv_sec) +		\
-			    (double)(t1.tv_usec - t0.tv_usec) * 1.0E-6)
-
-static double elapsed_since(crude_time t0)
+static double elapsed_sec(seconds t1, seconds t0)
 {
-     crude_time t1;
-     gettimeofday(&t1, 0);
-     return elapsed_sec(t1, t0);
+     return (double)(t1.tv_sec - t0.tv_sec) +
+	  (double)(t1.tv_usec - t0.tv_usec) * 1.0E-6;
 }
 
 #  define TIME_MIN_SEC 1.0e-2 /* from fftw2 */
+#  define HAVE_SECONDS_TIMER
+#endif
 
-#else /* !HAVE_GETTIMEOFDAY */
+#ifndef HAVE_SECONDS_TIMER
+#  include <time.h>
 
-/* Note that the only system where we are likely to need to fall back
-   on the clock() function is Windows, for which CLOCKS_PER_SEC is 1000
-   and thus the clock wraps once every 50 days.  This should hopefully
-   be longer than the time required to create any single plan! */
-crude_time X(get_crude_time)(void) { return clock(); }
+typedef clock_t seconds;
 
-#define elapsed_sec(t1,t0) ((double) ((t1) - (t0)) / CLOCKS_PER_SEC)
+static seconds getseconds(void) { return clock(); }
 
-static double elapsed_since(crude_time t0)
+static double elapsed_sec(seconds t1, seconds t0)
 {
-     return elapsed_sec(clock(), t0);
+     return ((double) (t1 - t0)) / CLOCKS_PER_SEC;
 }
 
 #  define TIME_MIN_SEC 2.0e-1 /* from fftw2 */
-
-#endif /* !HAVE_GETTIMEOFDAY */
-
-double X(elapsed_since)(const planner *plnr, const problem *p, crude_time t0)
-{
-     double t = elapsed_since(t0);
-     if (plnr->cost_hook)
-	  t = plnr->cost_hook(p, t, COST_MAX);
-     return t;
-}
+#  define HAVE_SECONDS_TIMER
+#endif
 
 #ifdef WITH_SLOW_TIMER
 /* excruciatingly slow; only use this if there is no choice! */
-typedef crude_time ticks;
-#  define getticks X(get_crude_time)
-#  define elapsed(t1,t0) elapsed_sec(t1,t0)
+typedef seconds ticks;
+#  define getticks getseconds
+#  define elapsed elapsed_sec
 #  define TIME_MIN TIME_MIN_SEC
 #  define TIME_REPEAT 4 /* from fftw2 */
 #  define HAVE_TICK_COUNTER
@@ -113,42 +121,47 @@ typedef crude_time ticks;
   }
 
 
-  double X(measure_execution_time)(const planner *plnr, 
-				   plan *pln, const problem *p)
+  double X(measure_execution_time)(plan *pln, const problem *p)
   {
+       seconds begin, now;
+       double t, tmax, tmin;
        int iter;
        int repeat;
 
-       X(plan_awake)(pln, AWAKE_ZERO);
+       AWAKE(pln, 1);
        p->adt->zero(p);
 
   start_over:
        for (iter = 1; iter; iter *= 2) {
-	    double tmin = 0;
-	    int first = 1;
-	    crude_time begin = X(get_crude_time)();
+	    tmin = 1.0E10;
+	    tmax = -1.0E10;
 
+	    begin = getseconds();
 	    /* repeat the measurement TIME_REPEAT times */
 	    for (repeat = 0; repeat < TIME_REPEAT; ++repeat) {
-		 double t = measure(pln, p, iter);
-		 
-		 if (plnr->cost_hook)
-		      t = plnr->cost_hook(p, t, COST_MAX);
+		 t = measure(pln, p, iter);
+
 		 if (t < 0)
 		      goto start_over;
 
-		 if (first || t < tmin)
+		 if (t < tmin)
 		      tmin = t;
-		 first = 0;
+		 if (t > tmax)
+		      tmax = t;
 
 		 /* do not run for too long */
-		 if (X(elapsed_since)(plnr, p, begin) > FFTW_TIME_LIMIT)
+		 now = getseconds();
+		 t = elapsed_sec(now, begin);
+
+		 if (t > FFTW_TIME_LIMIT)
 		      break;
 	    }
 
 	    if (tmin >= TIME_MIN) {
-		 X(plan_awake)(pln, SLEEPY);
-		 return tmin / (double) iter;
+		 tmin /= (double) iter;
+		 tmax /= (double) iter;
+		 AWAKE(pln, 0);
+		 return tmin;
 	    }
        }
        goto start_over; /* may happen if timer is screwed up */
@@ -156,10 +169,8 @@ typedef crude_time ticks;
 
 #else /* no cycle counter */
 
-  double X(measure_execution_time)(const planner *plnr, 
-				   plan *pln, const problem *p)
+  double X(measure_execution_time)(plan *pln, const problem *p)
   {
-       UNUSED(plnr);
        UNUSED(p);
        UNUSED(pln);
        return -1.0;
