@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,15 +18,20 @@
  *
  */
 
+/* $Id: conf.c,v 1.2 2003/03/15 20:29:43 stevenj Exp $ */
 
 #include "threads.h"
 
 static const solvtab s =
 {
+#ifdef HAVE_THREADS
+
      SOLVTAB(X(dft_thr_vrank_geq1_register)),
      SOLVTAB(X(rdft_thr_vrank_geq1_register)),
      SOLVTAB(X(rdft2_thr_vrank_geq1_register)),
 
+#endif /* HAVE_THREADS */
+
      SOLVTAB_END
 };
 
