@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-8 Matteo Frigo
- * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
+ * Copyright (c) 2003 Matteo Frigo
+ * Copyright (c) 2003 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,17 +23,17 @@
    f77api.c, possibly multiple times in order to support multiple
    compiler manglings (via redefinition of F77). */
 
-FFTW_VOIDFUNC F77(plan_with_nthreads, PLAN_WITH_NTHREADS)(int *nthreads)
+void F77(plan_with_nthreads, PLAN_WITH_NTHREADS)(int *nthreads)
 {
      X(plan_with_nthreads)(*nthreads);
 }
 
-FFTW_VOIDFUNC F77(init_threads, INIT_THREADS)(int *okay)
+void F77(init_threads, INIT_THREADS)(void)
 {
-     *okay = X(init_threads)();
+     X(init_threads)();
 }
 
-FFTW_VOIDFUNC F77(cleanup_threads, CLEANUP_THREADS)(void)
+void F77(cleanup_threads, CLEANUP_THREADS)(void)
 {
      X(cleanup_threads)();
 }
