@@ -1,6 +1,6 @@
 /* Re-use libbench2 and the test program, but override bench_main so that
    we can have different command-line syntax. */
-#include "my-getopt.h"
+#include "getopt.h"
 #include "bench.h"
 
 #include <stdio.h>
@@ -10,18 +10,11 @@
 #include <string.h>
 #include <time.h>
 
-#if defined(HAVE_THREADS) || defined(HAVE_OPENMP)
-#  define HAVE_SMP
-   extern int threads_ok;
-#endif
-
 #define CONCAT(prefix, name) prefix ## name
 #if defined(BENCHFFT_SINGLE)
 #define FFTW(x) CONCAT(fftwf_, x)
 #elif defined(BENCHFFT_LDOUBLE)
 #define FFTW(x) CONCAT(fftwl_, x)
-#elif defined(BENCHFFT_QUAD)
-#define FFTW(x) CONCAT(fftwq_, x)
 #else
 #define FFTW(x) CONCAT(fftw_, x)
 #endif
@@ -39,9 +32,9 @@ int verbose;
 
 static void do_problem(bench_problem *p)
 {
-     if (verbose)
+     if (verbose > 0)
 	  printf("PLANNING PROBLEM: %s\n", p->pstring);
-     /* BENCH_ASSERT(can_do(p)); */
+     BENCH_ASSERT(can_do(p));
      problem_alloc(p);
      setup(p);
      done(p);
@@ -69,32 +62,36 @@ static int prob_size_cmp(const void *p1_, const void *p2_)
      return (sz(*p1) - sz(*p2));
 }
 
-static struct my_option options[] =
+/* By default, don't allow fftw-wisdom tool to use threads; we don't
+   want users to put threads wisdom in the system wisdom, which would
+   break the wisdom for any non-threads program trying to use it. */
+#define USE_THREADS 0
+
+static struct option long_options[] =
 {
-  {"help", NOARG, 'h'},
-  {"version", NOARG, 'V'},
-  {"verbose", NOARG, 'v'},
+  {"help", no_argument, 0, 'h'},
+  {"version", optional_argument, 0, 'V'},
+  {"verbose", optional_argument, 0, 'v'},
 
-  {"canonical", NOARG, 'c'},
-  {"time-limit", REQARG, 't'},
+  {"canonical", no_argument, 0, 'c'},
+  {"time-limit", required_argument, 0, 't'},
 
-  {"output-file", REQARG, 'o'},
+  {"output-file", required_argument, 0, 'o'},
 
-  {"impatient", NOARG, 'i'},
-  {"measure", NOARG, 'm'},
-  {"estimate", NOARG, 'e'},
-  {"exhaustive", NOARG, 'x'},
+  {"impatient", no_argument, 0, 'i'},
+  {"estimate", no_argument, 0, 'e'},
+  {"exhaustive", no_argument, 0, 'x'},
 
-  {"no-system-wisdom", NOARG, 'n'},
-  {"wisdom-file", REQARG, 'w'},
+  {"no-system-wisdom", no_argument, 0, 'n'},
+  {"wisdom-file", required_argument, 0, 'w'},
 
-#ifdef HAVE_SMP
-  {"threads", REQARG, 'T'},
+#if USE_THREADS && defined(HAVE_THREADS)
+  {"threads", required_argument, 0, 'T'},
 #endif
 
   /* options to restrict configuration to rdft-only, etcetera? */
   
-  {0, NOARG, 0}
+  {0, no_argument, 0, 0}
 };
 
 static void help(FILE *f, const char *program_name)
@@ -111,12 +108,12 @@ static void help(FILE *f, const char *program_name)
  "              -c, --canonical: plan/optimize canonical set of sizes\n"
  "     -t <h>, --time-limit=<h>: time limit in hours (default: 0, no limit)\n"
  "  -o FILE, --output-file=FILE: output to FILE instead of stdout\n"
- "                -m, --measure: plan in MEASURE mode (PATIENT is default)\n"
+ "              -i, --impatient: plan in IMPATIENT mode (PATIENT is default)\n"
  "               -e, --estimate: plan in ESTIMATE mode (not recommended)\n"
  "             -x, --exhaustive: plan in EXHAUSTIVE mode (may be slow)\n"
  "       -n, --no-system-wisdom: don't read /etc/fftw/ system wisdom file\n"
  "  -w FILE, --wisdom-file=FILE: read wisdom from FILE (stdin if -)\n"
-#ifdef HAVE_SMP
+#if USE_THREADS && defined(HAVE_THREADS)
  "            -T N, --threads=N: plan with N threads\n"
 #endif
 	  "\nSize syntax: <type><inplace><direction><geometry>\n"
@@ -150,7 +147,9 @@ static char canonical_sizes[][32] = {
 int bench_main(int argc, char *argv[])
 {
      int c;
+     int ind;
      unsigned i;
+     char *short_options = make_short_options(long_options);
      int impatient = 0;
      int system_wisdom = 1;
      int canonical = 0;
@@ -159,19 +158,18 @@ int bench_main(int argc, char *argv[])
      char *output_fname = 0;
      bench_problem **problems = 0;
      int nproblems = 0, iproblem = 0;
-     time_t begin;
+     clock_t begin;
 
      verbose = 0;
      usewisdom = 0;
 
      bench_srand(1);
-#ifdef HAVE_SMP
-     /* do not configure FFTW with threads, unless the
-	user requests -T */
-     threads_ok = 0;
+#if USE_THREADS && defined(HAVE_THREADS)
+     BENCH_ASSERT(FFTW(init_threads)());
 #endif
 
-     while ((c = my_getopt(argc, argv, options)) != -1) {
+     while ((c = getopt_long (argc, argv, short_options,
+			      long_options, &ind)) != -1) {
 	  switch (c) {
 	      case 'h':
 		   help(stdout, argv[0]);
@@ -182,8 +180,8 @@ int bench_main(int argc, char *argv[])
 		   printf("fftw-wisdom tool for FFTW version " VERSION ".\n");
 		   printf(
 "\n"
-"Copyright (c) 2003, 2007-11 Matteo Frigo\n"
-"Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology\n"
+"Copyright (c) 2003 Matteo Frigo\n"
+"Copyright (c) 2003 Massachusetts Institute of Technology\n"
 "\n"
 "This program is free software; you can redistribute it and/or modify\n"
 "it under the terms of the GNU General Public License as published by\n"
@@ -203,7 +201,10 @@ int bench_main(int argc, char *argv[])
 		   break;
 		   
 	      case 'v':
-		   verbose = 1;
+		   if (optarg)
+			verbose = atoi(optarg);
+		   else
+			++verbose;
 		   break;
 		   
 	      case 'c':
@@ -211,23 +212,22 @@ int bench_main(int argc, char *argv[])
 		   break;
 
 	      case 't':
-		   hours = atof(my_optarg);
+		   hours = atof(optarg);
 		   break;
 
 	      case 'o':
 		   if (output_fname)
 			bench_free(output_fname);
 		   
-		   if (!strcmp(my_optarg, "-"))
+		   if (!strcmp(optarg, "-"))
 			output_fname = 0;
 		   else {
 			output_fname = (char *) bench_malloc(sizeof(char) *
-						    (strlen(my_optarg) + 1));
-			strcpy(output_fname, my_optarg);
+						    (strlen(optarg) + 1));
+			strcpy(output_fname, optarg);
 		   }
 		   break;
 
-	      case 'm':
 	      case 'i':
 		   impatient = 1;
 		   break;
@@ -246,15 +246,15 @@ int bench_main(int argc, char *argv[])
 
 	      case 'w': {
 		   FILE *w = stdin;
-		   if (strcmp(my_optarg, "-") && !(w = fopen(my_optarg, "r"))) {
+		   if (strcmp(optarg, "-") && !(w = fopen(optarg, "r"))) {
 			fprintf(stderr,
-				"fftw-wisdom: error opening \"%s\": ", my_optarg);
+				"fftw-wisdom: error opening \"%s\"", optarg);
 			perror("");
 			exit(EXIT_FAILURE);
 		   }
 		   if (!FFTW(import_wisdom_from_file)(w)) {
-			fprintf(stderr, "fftw_wisdom: error reading wisdom "
-				"from \"%s\"\n", my_optarg);
+			fprintf(stderr, "fftw_wisdom: error reading wisdom"
+				"from \"%s\"\n", optarg);
 			exit(EXIT_FAILURE);
 		   }
 		   if (w != stdin)
@@ -262,19 +262,20 @@ int bench_main(int argc, char *argv[])
 		   break;
 	      }
 
-#ifdef HAVE_SMP
+#if USE_THREADS && defined(HAVE_THREADS)
 	      case 'T':
-		   nthreads = atoi(my_optarg);
-		   if (nthreads < 1) nthreads = 1;
-		   threads_ok = 1;
-		   BENCH_ASSERT(FFTW(init_threads)());
+		   nthreads = atoi(optarg);
+		   if (nthreads > 1) {
+			fprintf(stderr, "fftw-wisdom: "
+				"not compiled with thread support\n");
+			exit(EXIT_FAILURE);
+		   }
 		   break;
 #endif
 
 	      case '?':
-		   /* `my_getopt' already printed an error message. */
-		   cleanup();
-		   return EXIT_FAILURE;
+		   /* `getopt_long' already printed an error message. */
+		   break;
 
 	      default:
 		   abort ();
@@ -299,31 +300,41 @@ int bench_main(int argc, char *argv[])
 		    char ps[64];
 		    if (!strchr(canonical_sizes[i],'x')
 			|| !strchr(types[j],'o')) {
-#ifdef HAVE_SNPRINTF
-			 snprintf(ps, sizeof(ps), "%s%s", types[j], canonical_sizes[i]);
-#else
 			 sprintf(ps, "%s%s", types[j], canonical_sizes[i]);
-#endif
 			 add_problem(ps, &problems, &iproblem, &nproblems);
 		    }
 	       }
 	  }
      }
 
-     while (my_optind < argc) {
-	  if (!strcmp(argv[my_optind], "-")) {
+     while (optind < argc) {
+	  if (!strcmp(argv[optind], "-")) {
 	       char s[1025];
 	       while (1 == fscanf(stdin, "%1024s", s))
 		    add_problem(s, &problems, &iproblem, &nproblems);
 	  }
 	  else
-	       add_problem(argv[my_optind], &problems, &iproblem, &nproblems);
-	  ++my_optind;
+	       add_problem(argv[optind], &problems, &iproblem, &nproblems);
+	  ++optind;
      }
 
      nproblems = iproblem;
      qsort(problems, nproblems, sizeof(bench_problem *), prob_size_cmp);
 
+     begin = clock();
+     for (iproblem = 0; iproblem < nproblems; ++iproblem) {
+	  if (hours <= 0
+	      || hours > (clock() - begin) / 3600.0 / CLOCKS_PER_SEC)
+	       do_problem(problems[iproblem]);
+	  problem_destroy(problems[iproblem]);
+	  
+     }
+     free(problems);
+     
+     if (verbose && hours > 0
+	 && hours < (clock() - begin) / 3600.0 / CLOCKS_PER_SEC)
+	  fprintf(stderr, "EXCEEDED TIME LIMIT OF %g HOURS.\n", hours);
+
      if (!output_fname)
 	  output_file = stdout;
      else
@@ -334,20 +345,6 @@ int bench_main(int argc, char *argv[])
 	       exit(EXIT_FAILURE);
 	  }
 
-     begin = time((time_t*)0);
-     for (iproblem = 0; iproblem < nproblems; ++iproblem) {
-	  if (hours <= 0
-	      || hours > (time((time_t*)0) - begin) / 3600.0)
-	       do_problem(problems[iproblem]);
-	  problem_destroy(problems[iproblem]);
-	  
-     }
-     free(problems);
-     
-     if (verbose && hours > 0
-	 && hours < (time((time_t*)0) - begin) / 3600.0)
-	  fprintf(stderr, "EXCEEDED TIME LIMIT OF %g HOURS.\n", hours);
-
      FFTW(export_wisdom_to_file)(output_file);
      if (output_file != stdout)
 	  fclose(output_file);
@@ -355,6 +352,7 @@ int bench_main(int argc, char *argv[])
 	  bench_free(output_fname);
 
      cleanup();
+     bench_free(short_options);
 
      return EXIT_SUCCESS;
 }
